unit PGPFuncs;

// 08.08.2000: NULL (=#0) string passed to KeyRingID/KeyProps (for SPGP 2.5.X)
// 14.02.2000: TabExtract changed by adding Trim() to strip off any left CRLF
// 16.11.1999: AddSelKeys updated using the new KeyRemove function
// 25.09.1999: String handling changed for KeyRingID (no TAB before CRLF)

interface

uses
  Windows, SysUtils, Classes, StdCtrls, SPGP;

const
  E = '';
  SP = ' ';
  TAB = #9;
  NULL = #0;
  COMMA = ';';
  LF = #10;
  CR = #13;
  NL1 = #13#10;
  NL2 = NL1 + NL1;
  NL3 = NL1 + NL2;
  BUFSIZE = 10240;

type
  TType = (PGPPublic, PGPPrivate);
  TKeyType = Array[TType] of String;
  TKeyTrust = Array[PGPKeyTrust_Undefined..PGPKeyTrust_Ultimate] of String;
  TKeyValidity = Array[PGPValidity_Unknown..PGPValidity_Complete] of String;
  TKeyAlgorithm = Array[PGPPublicKeyAlgorithm_Invalid..PGPPublicKeyAlgorithm_DSA] of String;
  TKeyResult = (krID, krAdd, krExport, krPubRing, krFingerprint);
  TKeyRec = Record
   kType: String;
   kID: String;
   kUser: String;
   kSize: String;
   kDateTime: String;
   kGMT: String;
   kFingerprint: String;
   kTrust: String;
   kValidity: String;
   kAlgorithm: String;
  end;

const
    aKeyType: TKeyType = (
      'PGPKey_Public',
      'PGPKey_Private'
      );
    aKeyTrust: TKeyTrust = (
      'PGPKeyTrust_Undefined',
      'PGPKeyTrust_Unknown',
      'PGPKeyTrust_Never',
      '',
      '',
      'PGPKeyTrust_Marginal',
      'PGPKeyTrust_Complete',
      'PGPKeyTrust_Ultimate'
      );
    aKeyValidity: TKeyValidity = (
      'PGPValidity_Unknown',
      'PGPValidity_Invalid',
      'PGPValidity_Marginal',
      'PGPValidity_Complete'
      );
    aKeyAlgorithm: TKeyAlgorithm = (
      'PGPPublicKeyAlgorithm_Invalid',
      'PGPPublicKeyAlgorithm_RSA',
      'PGPPublicKeyAlgorithm_RSAEncryptOnly',
      'PGPPublicKeyAlgorithm_RSASignOnly',
      'PGPPublicKeyAlgorithm_ElGamal',
      'PGPPublicKeyAlgorithm_DSA'
      );

function GetKeyLists(const KeyFile: String; const NameList, IDList: TStringList): Integer;
function GetFingerprint(const User: String; var Fingerprint: String): Integer;
function ExtractKey(var User: String; var Key: String): Integer;
function AddSelKeys(const KeyFile: String; const KeyList: TListBox;
		    const NameList, IDList: TStringList; var KeyInfo: String): Integer;

implementation

function FPSort(const sFingerprint: String): String;
var iPos: Integer;
begin
  Result:=E;
  if sFingerprint<>E then begin
    iPos:=1;
    while iPos<Length(sFingerprint) do begin
      if Result=E then
	Result:=Copy(sFingerprint, iPos, 4)
      else Result:=Result + SP + SP + Copy(sFingerprint, iPos, 4);
      inc(iPos, 4);
    end;
  end
  else Result:=E;
end;

function ExtractStr(var Source: String; Pos, Len: Integer): String;
begin
  Result:=Copy(Source, Pos, Len);
  Delete(Source, Pos, Len);
end;

function TabExtract(var sProps: String): String;
var iTABPos: Integer; sBuffer: String;
begin
  Result:=E;
  if sProps<>E then begin
    iTABPos:=pos(TAB, sProps);
    if iTABPos<>0 then begin
      sBuffer:=ExtractStr(sProps, 1, iTABPos);
      Result:=Copy(sBuffer, 1, pred(iTABPos));
    end
    else begin
      Result:=Trim(sProps);
      sProps:=E;
    end;
  end;
end;

function GetKeyProps(var sKeyInfo: String): String;
var iCRLFPos: Integer;
begin
  iCRLFPos:=pos(NL1, sKeyInfo);
  if iCRLFPos<>0 then begin
    Result:=ExtractStr(sKeyInfo, 1, succ(iCRLFPos));
    Delete(Result, iCRLFPos, 1);
    Result[Length(Result)]:=TAB;
  end
  else begin
    Result:=sKeyInfo;
    sKeyInfo:=E;
  end;
end;

function GetKeyResult(var sKeyInfo: String; krKind: TKeyResult; var iError: Integer): String;
var sKeyProps: String; rKey: TKeyRec;
begin
  Result:=E;
  FillChar(rKey, SizeOf(TKeyRec), 0);
  sKeyProps:=GetKeyProps(sKeyInfo);
  if sKeyProps<>E then with rKey do try
    kType:=aKeyType[TType(StrToInt(TabExtract(sKeyProps)))];
    kID:=TabExtract(sKeyProps);
    kUser:=TabExtract(sKeyProps);
    if (krKind<>krID) and (krKind<>krPubring) then begin
      kSize:=TabExtract(sKeyProps)+ ' bits';
      kDateTime:=TabExtract(sKeyProps);
      kGMT:=TabExtract(sKeyProps);
      kFingerprint:=TabExtract(sKeyProps);
      kTrust:=aKeyTrust[StrToInt(TabExtract(sKeyProps))];
      kValidity:=aKeyValidity[StrToInt(TabExtract(sKeyProps))];
      kAlgorithm:=aKeyAlgorithm[StrToInt(TabExtract(sKeyProps))];
    end;
    case krKind of
      krID: Result:=kID;
      krAdd: Result:=kUser + NL1 + kAlgorithm + NL2 + kValidity + NL1 + kTrust + NL1 + kType;
      krExport: Result:=kUser + COMMA + SP + kType + COMMA + SP + kSize;
      krPubRing: Result:=kID + LF + kUser;
      krFingerprint: Result:=kUser + NL2 + kID + NL2 + kAlgorithm + NL2 + FPSort(kFingerprint);
    end;
  except
    iError:=-1;
  end;
end;

function GetPubKeys(const KeyFile: String; var KeyList: TStringList): Integer;
var sProps, sResult: String; iBufSize: Integer;
begin
  sProps:=NULL;									// adjustment for SPGP 2.5.X
  sResult:=E;
  if KeyFile=E then
    Result:=KeyRingCount
  else Result:=1;
  if Result>0 then begin
    iBufSize:=Result * 256;
    repeat
      if KeyFile=E then
        Result:=KeyRingID(sProps, iBufSize)
      else Result:=_KeyImportFile(KeyFile, sProps, iBufSize, false, false);
      iBufSize:=Result;
    until Result<=0;
    while (Result=0) and (sProps<>E) do begin
      if sResult=E then
        sResult:=GetKeyResult(sProps, krPubRing, Result)
      else sResult:=sResult + LF + GetKeyResult(sProps, krPubRing, Result);
    end;
  end;
  if sResult<>E then
    KeyList.Text:=sResult
  else Result:=-1;
end;

function GetKeyLists(const KeyFile: String; const NameList, IDList: TStringList): Integer;
var slNewBuf, slNameBuf, slIDBuf: TStringList; iIndex: Integer;
begin
  slNewBuf:=TStringList.Create;
  slNameBuf:=TStringList.Create;
  slIDBuf:=TStringList.Create;
  slNameBuf.Assign(NameList);
  slIDBuf.Assign(IDList);
  if Assigned(IDList) then IDList.Clear;
  if Assigned(NameList) then NameList.Clear;
  with slNewBuf do try
    Result:=GetPubKeys(KeyFile, slNewBuf);
    if Result=0 then begin
      iIndex:=0;
      while iIndex<Count do begin
        // make sure to not add keys that already are on the ring
        // otherwise they will be deleted by KeyRemove (s. below)
        if (KeyFile=E) or ((slNameBuf.IndexOf(Strings[iIndex])=-1) and (slIDBuf.IndexOf(Strings[iIndex])=-1)) then begin
          if not odd(iIndex) then begin
            if Assigned(IDList) then IDList.Add(Strings[iIndex])
          end
          else begin
            if Assigned(NameList) then NameList.Add(Strings[iIndex]);
          end;
        end;
	inc(iIndex);
      end;
    end;
  finally
    slNewBuf.Free;
    slNameBuf.Free;
    slIDBuf.Free;
  end;
end;

function ExtractKey(var User: String; var Key: String): Integer;
var sProps: String; iBufSize: Integer;
begin
  sProps:=NULL;									// adjustment for SPGP 2.5.X
  iBufSize:=1024;
  repeat
    Result:=KeyExport(User, Key, iBufSize, false, true);
    iBufSize:=Result;
  until Result<=0;
  if Result=0 then begin
    iBufSize:=256;
    repeat
      Result:=KeyProps(User, sProps, iBufSize);
      iBufSize:=Result;
    until Result<=0;
    User:=GetKeyResult(sProps, krExport, Result);
  end;
  if (Result=0) and ((User=E) or (Key=E)) then Result:=-1;
end;

function GetFingerprint(const User: String; var Fingerprint: String): Integer;
var sProps: String; iBufSize: Integer;
begin
  sProps:=NULL;									// adjustment for SPGP 2.5.X
  iBufSize:=256;
  repeat
    Result:=KeyProps(User, sProps, iBufSize);
    iBufSize:=Result;
  until Result<=0;
  if Result=0 then Fingerprint:=GetKeyResult(sProps, krFingerprint, Result) + NL2;
  if Fingerprint=E then Result:=-1;
end;

function GetIDList(var IDList: String): Integer;
var slBuffer: TStringList;
begin
  Result:=0;
  IDList:=E;
  try
    slBuffer:=TStringList.Create;
    try
      GetKeyLists(E, nil, slBuffer);
      IDList:=slBuffer.Text;
    finally
      slBuffer.Free;
    end;
  except
    Result:=-1;
  end;
end;

function ExecAdd(const KeyFile, SelKeys: String; var KeyInfo: String): Integer;
var sProps: String; iBufSize: Integer;
begin
  Result:=0;
  KeyInfo:=E;
  iBufSize:=256;
  repeat
    if KeyFile<>E then
      Result:=KeyImportFile(KeyFile, sProps, iBufSize)
    else if SelKeys<>E then Result:=KeyImport(SelKeys, sProps, iBufSize);
    iBufSize:=Result;
  until Result<=0;
  if Result=-11995 then Result:=0;
end;

{New version using KeyRemove}
function AddSelKeys(const KeyFile: String; const KeyList: TListBox;
		    const NameList, IDList: TStringList; var KeyInfo: String): Integer;
var
  iIndex: Integer;
  sResult, sKeyIDs, sKeyNames: String;
begin
  KeyInfo:=E;
  with KeyList do begin
    Result:=ExecAdd(KeyFile, E, sResult);
    if (Result=0) and (SelCount<>0) then begin
      sKeyIDs:=E;
      sKeyNames:=E;
      for iIndex:=0 to pred(Items.Count) do begin
        if Selected[iIndex] then begin
          if sKeyNames=E then
            sKeyNames:=NameList[iIndex]
          else sKeyNames:=sKeyNames + NL1 + NameList[iIndex];
        end
        else begin
          if sKeyIDs=E then
            sKeyIDs:=IDList[iIndex]
          else sKeyIDs:=sKeyIDs + ',' + IDList[iIndex];
        end;
      end;
      // be careful with removing keys automatically to
      // not remove keys that were not added (s. above)
      if sKeyIDs<>E then Result:=KeyRemove(sKeyIDs);
      if Result=0 then KeyInfo:=sKeyNames;
    end;
  end;
end;

end.

